@@ -60,13 +60,23 @@ const char* convert_charset (const char* s)
 {
 	static char buffer[PATH_MAX];
 	static wchar_t wbuf[PATH_MAX];
-	if (mbstowcs (wbuf, s, sizeof(buffer) - 1) == (size_t)-1) return s;
+	char *locale = setlocale(LC_CTYPE, "");
 	size_t i = 0, o = 0;
+
+	if (mbstowcs(wbuf, s, sizeof(buffer) - 1) == (size_t)-1) {
+		setlocale(LC_CTYPE, locale);
+		return s;
+	}
 	while (wbuf[i]) {
-		if (wbuf[i] < 256) buffer[o] = wbuf[i]; else buffer[o] = '?';
-		++i; ++o;
+		if (wbuf[i] < 256)
+			buffer[o] = wbuf[i];
+		else
+			buffer[o] = '?';
+		++i;
+		++o;
 	}
 	buffer[o] = 0;
+	setlocale(LC_CTYPE, locale);
 	return buffer;
 }
 
@@ -78,7 +88,9 @@ static void server_shutdown(int signal)
 
 static void kill_children(int signal)
 {
+	int old_errno = errno;
 	while(waitpid(0, NULL, WNOHANG) > 0) ;
+	errno = old_errno;
 }
 
 static int parse_config(void)
@@ -285,6 +297,7 @@ static void do_daemon(void)
 	struct sockaddr_in pin;
 	socklen_t addrlen;
 	fd_set set;
+	FILE *pidfile;
 
 	signal(SIGINT, server_shutdown);
 	signal(SIGCHLD, kill_children);
@@ -293,6 +306,13 @@ static void do_daemon(void)
 		openlog("kissd", LOG_PID, LOG_DAEMON);
 	log("%s", "starting kissd...");
 
+	if ((pidfile = fopen("/var/run/kissd.pid", "w")) != NULL) {
+		fprintf(pidfile, "%ld\n", (long) getpid());
+		fclose(pidfile);
+	}
+	else
+		log("fopen /var/run/kissd.pid: %s", strerror(errno));
+
 	create_socket(&sd_pclink, 	 PORT_PCLINK, SOCK_STREAM);
 	create_socket(&sd_udp_responder, PORT_PCLINK, SOCK_DGRAM);
 
@@ -384,7 +404,6 @@ int main(int argc, char *argv[])
 		{ NULL, 0, NULL, 0 }
 	};
 
-	setlocale (LC_ALL, "");
 	while (1) {
 		c = getopt_long(argc, argv, "ac:dhkv", long_options, NULL);
 		if (c == -1)
